#!/usr/bin/env python3
#
#-*- coding: utf-8 -*-
#
# ll2mh-p3.el.py
#
##########################################################################
#
# Copyright:
#
# G1OGY 20081102  Initial release of gpsdll2mh.py
#
# G1OGY 20180110: `convert` to standalone, manual entry 
# (its taken only 10 years...)
#
# G1OGY 20221101: upgrade to run against python3
#
# G1OGY 20221226: support direct paste of lat/long from (say) Google Maps.
#                 add Elevation (MASL) via open-elevation.com SRTM data.
#                 NOTE: SRTM data does not cover the whole globe:
#                       both Sea Level and `no data` will return ZERO.
#
##########################################################################
#
# Description:
#
# ll2mh-p3.py  prints Amateur Radio Maidenhead LOCATOR at user-determined,
# arbitary, even-number precision (max 52) derived from manually
# entered decimal LAT/LONG co-ordinates.
#
# Note that anything over 12-char is practically useless until there
# is software to use it. (WiP - another 10 years...?)
#
##########################################################################
#
# Credits:
# The Lat/Long to Maidenhead part of this program is a rework of an 
# original script by Dan Jacobson -- http://jidanni.org/geo/maidenhead/ ;
# latterly updated by Rene Kanter.
#
## lonlat2maidenhead -- long/lat to Maidenhead grid calculator not limited
## to 6 characters.
## Copyright       : http://www.fsf.org/copyleft/gpl.html
## Author          : Dan Jacobson -- http://jidanni.org/geo/maidenhead/
## Created On      : Sat Mar 15 03:54:08 2003
## rkanters 2004.2.20 version ll2mh
#
# Thank you, gentlemen.
#
##########################################################################
#
# Licence:
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
#      http://www.gnu.org/licenses/gpl.html
#
#
##########################################################################
#
##########################################################################
# 
#       

# get modules and test runtime
import  sys
if sys.version_info[0]!=3 or sys.version_info[1]<9:
     print("This script requires Python version 3.7 or greater")
     sys.exit(1)

import re, string, json
from urllib.request import urlopen

def is_openelev_available():
    try:
        urlopen('https://api.open-elevation.com/api/v1/lookup?locations=10.0,10.0', timeout=2)
        return True
    except:
        return False

# initialise null Elevation value
# (if open-elevation is unavailable, returns WWL only)     
E = "NaN"
     
# WWL precision
while True:
     try:
          loclen=int(input("\nEnter LOCATOR precision required [4/6/10/12/--/50!]\n(even Number!) : "))
     except ValueError:
          print("invalid input!")
          continue
     else:
          break
if (int(loclen) % 2 == 0) and (int(loclen) < 52): 
    pass
else:
    loclen=12
    print("\nInvalid input! Proceding with default value (12)")


print("Enter LAT.ITUDE [<space> or <comma>] LONG.ITUDE (Decimal)")
line=sys.stdin.readline()
if re.search(',', line):
     line = re.sub(","," ", line)
else:
     pass

ll=re.findall(r'([-0-9.]+)\s+([-0-9.]+)',line)


# get elevation of lat/long point from api.open-elevation.com
for a,b in ll:
     elat=str(a)
     elong=str(b)
     eline=elat + ',' + elong
     
if is_openelev_available() == False:
     pass
else:
     url = "https://api.open-elevation.com/api/v1/lookup?locations=" + eline
     with urlopen(url, timeout=10) as e:
          resp = json.load(e)
          E = resp['results'][0]['elevation']

     
# initialise WWL calcs
A = ord('A')
maxn=int(loclen)/2

# run
if ll:
    for x,y in ll:
        glat=float(x)
        glon=float(y)
else:
    sys.stderr.write(sys.argv[0]+': ERROR :: Cannot determine LAT / LONG. Input Decimal Coordinates \n\n')
    sys.exit(44)

if -180<=glon<180:
    pass
else:
    sys.stderr.write('ERROR :: longitude must be +/- 180 degrees, MAX\n\n')
    sys.exit(32)
    
if -90<=glat<90:
    pass
else:
    sys.stderr.write('ERROR :: latitude must be +/- 90 degrees, MAX\n\n')
    sys.exit(33)                      # can't handle North pole, sorry, [A-R]
    
glon=(glon+180.0)/20                  # scale down and set up for first digit
glat=(glat+90.0)/10
mhloc=""
i=0

while i<maxn:
    i+=1
    loni=int(glon)
    lati=int(glat)
    if i%2:
        mhloc+=chr(A+loni)+chr(A+lati)
        glon=(glon-loni)*10
        glat=(glat-lati)*10
    else:
        mhloc+=str(loni)+str(lati)
        glon=(glon-loni)*24
        glat=(glat-lati)*24


# print lat/long for input confirmation 

print('\n\n','Lat / Long: ', line,'\n','Locator:    ', mhloc,'\n','MASL:       ',E, '\n')

# Close down                        
# "returns the grid square, to the precision given, that contains the given point."
#       -- Dan J
# 
#
#
##########################################################################
# Things to do
#
# GUIfication - Tk??

